/* This file is auto generated, version 1 */
/* SMP */
#define UTS_MACHINE "x86_64"
#define UTS_VERSION "#1 SMP Mon, 31 Aug 2026 14:22:00 +0000"
#define LINUX_COMPILE_BY "linux-lts515"
#define LINUX_COMPILE_HOST "archlinux"
#define LINUX_COMPILER "gcc (GCC) 16.2.1 20260810, GNU ld (GNU Binutils) 2.47"
